# Launchpad reports

Convenient web-based frontend for Launchpad


## Installation

LP reports is built on top of Python, Flask and MongoDB.

### Development environment

1. Install system requirements:
    ```shell
    $ sudo apt-get install build-essential python-dev python-pip mongodb
    $ sudo pip install virtualenv
    ```

2. Install Python requirements:
    ```shell
    $ cd lp-reports
    $ virtualenv .venv
    $ source .venv/bin/activate
    $ pip install -r requirements.txt
    ```

3. Make sure that MongoDB is running:
    ```shell
    $ sudo service mongodb start
    ```

4. Generate Launchpad credentials:
   ```shell
   $ pip install launchpadlib
   $ python manage.py generate-lp-credentials
   ```
   Copy output of the command above.

5. Create local configuration file `~/.lpreports.conf` in your home folder and
   populate it with output from step #4. Also add following config to the file:
   ```
   GOOGLE_ID = "804310106349-kbvmtnnlet8936o6e6l63p8vfl7uqi2i." \
                "apps.googleusercontent.com"
   GOOGLE_SECRET = "BmprHAgvag0_PDgS-4S574sb"
   ```

   These options allow to run google oauth on your localhost. If you want to
   create your own credentials for Google oauth, go to
   https://console.developers.google.com "APIs & Auth" -> Credentials and create
   new credentials there.

6. Populate database with teams:
   ```shell
   $ python manage.py collect-assignees
   ```

7. Sync the database with Launchpad:
   ```shell
   $ python manage.py syncdb
   ```

8. Start your local development server:
   ```shell
   $ python manage.py devserver
   ```


### Deployment to production with NGINX

1. Install system requirements:
   ```shell
   $ sudo apt-get install build-essential python-dev python-pip mongodb
   $ sudo apt-get install nginx uwsgi uwsgi-plugin-python
   $ sudo apt-get install libffi-dev libssl-dev
   $ sudo pip install virtualenv
   ```

2. Create service user:
   ```shell
   $ sudo adduser lpreports
   ```

3. Create necessary directories:
   ```shell
   $ sudo mkdir -p /var/www
   $ sudo mkdir -p /var/run/uwsgi/lpreports
   $ sudo mkdir -p /var/log/lpreports
   $ sudo mkdir -p /etc/lpreports
   $ sudo chown -R lpreports:lpreports /var/run/uwsgi/lpreports
   $ sudo chown -R lpreports:lpreports /var/log/lpreports
   ```

4. Clone project. You will need to request private SSH key for service account
   in gerrit (review.fuel-infra.org):
   ```shell
   $ cd /var/www
   $ sudo git clone ssh://lp-reports-bot@review.fuel-infra.org:29418/tools/lp-reports
   $ sudo chown -R lpreports:lpreports /var/www/lp-reports
   ```

5. Create and populate configuration file:
   ```shell
   $ sudo touch /etc/lpreports/lpreports.conf
   ```

   It should contain following options:
   * SECRET_KEY
   * LP_CONSUMER_KEY
   * LP_TOKEN
   * LP_SIGNATURE
   * GOOGLE_ID
   * GOOGLE_SECRET

5.1 Launchpad credentials
    LP_CONSUMER_KEY, LP_TOKEN and LP_SIGNATURE can be obtained by running
    following command on your local installation of lp-reports (do not run this
    command on server, it will try to open a browser there):

    ```shell
    $ pip install launchpadlib
    $ python manage.py generate-lp-credentials
    ```

    Note: you should be logged in as launchpadbot@mirantis.com in Launchpad.
    Password for this account is reflected in https://mirantis.jira.com/browse/IT-22937

5.2 Google credentials
    We're using Google oauth for authentication in lp-reports. To generate new
    credentials browse to https://console.developers.google.com, navigate to
    "APIs & Auth", "Credentials" and create new "OAuth 2.0 client ID" there.

    Note: Google requires domain names for everything except localhost. If you
    don't have domain name, just update /etc/hosts on the server and on your local
    machine and use this name in Google developer console.

5.3 SECRET_KEY
    This options is used to sign cookies and other things. Please make sure that
    you're not using the default value!

6. Setup virtual environment:
   ```shell
   $ sudo su lpreports
   $ cd /var/www/lp-reports
   $ virtualenv .venv
   $ source .venv/bin/activate
   $ pip install -r requirements.txt
   $ pip install pyopenssl ndg-httpsclient pyasn1 # this is a workaround for Python version older than 2.7.6 to get rid of warning about insecure channel
   $ deactivate
   ```

7. uwsgi configuration:
   Create file lpreports.ini at /etc/uwsgi/apps-available with the following
   content:
   ```
   [uwsgi]
   vhost = true

   venv = /var/www/lp-reports/.venv
   chdir = /var/www/lp-reports

   module = wsgi
   callable = app

   master = True
   processes = 5

   socket = /run/uwsgi/app/lpreports/socket
   chmod-socket = 666
   vacuum = True

   die-on-term = true

   logto=/var/log/lpreports/uwsgi.log
   ```

   Create a symlink to this file in /etc/uwsgi/apps-enabled.

   Restart uwsgi service to make sure that configuration is correct:
   ```shell
   $ sudo service uwsgi restart
   ```

8. nginx configuration:
   Create file lpreports.conf in /etc/nginx/sites-available with the following
   content:
   ```
   server {
       listen 80;
       server_name lp-reports-staging.vm.mirantis.net;

       error_log /var/log/lpreports/nginx.error.log;
       access_log /var/log/lpreports/nginx.access.log;

       gzip             on;
       gzip_comp_level  2;
       gzip_min_length  1000;
       gzip_proxied     expired no-cache no-store private auth;
       gzip_types       text/plain application/x-javascript text/xml text/css application/xml;

       location / {
           include uwsgi_params;
           uwsgi_pass unix:/run/uwsgi/app/lpreports/socket;
       }

       location /static {
           alias /var/www/lp-reports/lpreports/static;
       }

   }
   ```

   Don't forget to update server_name.

   Create a symlink to this file in /etc/nginx/sites-enabled.

   Run following command to check that nginx configuration is correct:
   ```shell
   $ sudo nginx -t
   ```

   Restart nginx:
   ```shell
   $ sudo service nginx restart
   ```

9. Populate the database:
   ```shell
   $ cd /var/www/lp-reports
   $ source .venv/bin/activate
   $ python manage.py collect-assignees
   $ python manage.py syncdb # this operation might take up to two hours
   ```

10. Setup crontab:
    Change current loging to user lpreports and add following to crontab:
    ```
    */10 * * * * /var/www/lp-reports/.venv/bin/python /var/www/lp-reports/manage.py syncdb > /var/log/lpreports/syncdb.log
    */1 * * * *  /var/www/lp-reports/.venv/bin/python /var/www/lp-reports/manage.py collect-assignees > /var/log/lpreports/collect_assignees.log
    0 22 * * *   /var/www/lp-reports/.venv/bin/python /var/www/lp-reports/manage.py cleanup-db > /var/log/lpreports/cleanup_db.log
    ```
