import os
import unittest

import requests
import yaml


ENDPOINTS = [
    '/',
    '/stats/project/fuel/',
    '/stats/project/fuel/6.1',
    '/stats/project/fuel/7.0',
    '/stats/project/mos/6.1',
    '/stats/project/mos/7.0',
    '/stats/project/fuelplusmos/6.1/',
    '/stats/project/fuelplusmos/7.0/',

    '/reports/sla_report?milestone=',
    '/reports/sla_report?milestone=6.0.1',
    '/reports/sla_report?milestone=6.1',
    '/reports/hcf_status',
    '/reports/hcf_status?milestone=6.0.1',
    '/reports/hcf_status?milestone=6.1',
    '/reports/custom_report',
    '/reports/custom_report?milestone=6.0.1',
    '/reports/custom_report?milestone=6.1',
    '/reports/fuel_triage_queue',
    '/reports/mos_linux_triage_queue',
    '/reports/mos_openstack_triage_queue',
    '/reports/partners_triage_queue',
    '/reports/docs_triage_queue',

    '/api/incoming_outgoing/7.0',
    '/api/incoming_outgoing/7.0/wednesday',
]


class TestApp(unittest.TestCase):
    def _read_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'tests.yaml')
        with open(conf_file) as fl:
            conf = yaml.load(fl)

        self.host = conf['host']
        self.port = conf['port']

    def setUp(self):
        super(TestApp, self).setUp()

        self._read_config()

        # do print to separate output of different tests
        print()

    def make_url(self, endpoint):
        return 'http://%s:%d%s' % (self.host, self.port, endpoint)

    def send_get(self, url):
        resp = requests.get(url)
        if resp.status_code != 200:
            raise RuntimeError('Server returned code %s on get request'
                               'to url %s' % (resp.status_code, url))
        return resp


class TestEndpoints(TestApp):
    def test_endpoints(self):
        for endpoint in ENDPOINTS:
            self.send_get(self.make_url(endpoint))
