import datetime

import bs4

import test_app


CLASS_LST = [
    'statusNEW',
    'statusINCOMPLETE',
    'statusOPINION',
    'statusINVALID',
    'statusWONTFIX',
    'statusCONFIRMED',
    'statusTRIAGED',
    'statusINPROGRESS',
    'statusFIXCOMMITTED',
    'statusFIXRELEASED'
]


def map_class_name(name):
    return name[len('status'):].lower()


class TestCharts(test_app.TestApp):
    def _compare(self, state, a, b):
        if a != b:
            self.diff_counter += abs(a - b)
            print("%s: expected %d, but got %d" % (state, b, a))

    def _find_real_numbers(self, project, milestone):
        resp = self.send_get('https://launchpad.net/%s/+milestone/%s' %
                             (project, milestone))
        doc = bs4.BeautifulSoup(resp.text)
        activities = doc.find('div', attrs={'id': 'milestone-activities'})

        real_numbers = {}

        for class_name in CLASS_LST:
            status_name = map_class_name(class_name)

            elem = activities.find('span', attrs={'class': class_name})

            if elem:
                real_numbers[status_name] = int(elem.strong.text)
            else:
                real_numbers[status_name] = 0

        return real_numbers

    def _get_chart_last_day_data(self, chart_url, *args):
        url = self.make_url(chart_url % args)

        resp = self.send_get(url)

        chart_numbers = {}
        for dct in resp.json():
            chart_numbers[dct['key']] = dct['values'][-1][1]

        return chart_numbers

    def _verify_numbers(self, real_numbers, chart_numbers):
        self.diff_counter = 0

        self._compare('Incomplete',
                     chart_numbers['Incomplete'],
                     real_numbers['incomplete'])

        self._compare('Open',
                     chart_numbers['Open'],
                     real_numbers['confirmed'] + real_numbers['triaged'])

        self._compare('In Progress',
                     chart_numbers['In Progress'],
                     real_numbers['inprogress'])

        self._compare('Resolved',
                     chart_numbers['Resolved'],
                     real_numbers['fixcommitted'] +
                     real_numbers['wontfix'] +
                     real_numbers['invalid'] +
                     real_numbers['opinion'])

        self._compare('Verified',
                     chart_numbers['Verified'],
                     real_numbers['fixreleased'])

        print('Total sum of differences: %d' % self.diff_counter)

    # def _verify_chart(self, project, milestone):
    #     real_numbers = self._find_real_numbers(project, milestone)
    #
    #     chart_numbers = self._get_chart_last_day_data(
    #         '/project/%s/api/release_chart_trends/%s/get_data',
    #         project, milestone)
    #
    #     self._verify_numbers(real_numbers, chart_numbers)
    #
    # def test_mos_601_chart(self):
    #     print("Testing MOS chart for 6.0.1")
    #     self._verify_chart('mos', '6.0.1')
    #
    # def test_fuel_601_chart(self):
    #     print("Testing Fuel chart for 6.0.1")
    #     self._verify_chart('fuel', '6.0.1')
    #
    # def test_fuel_61_chart(self):
    #     print("Testing Fuel chart for 6.1")
    #     self._verify_chart('fuel', '6.1')

    def _test_fuelmos_chart(self, milestone):
        print("Testing Fuel+MOS chart for %s" % milestone)

        real_numbers = self._find_real_numbers('fuel', milestone)
        real_numbers_mos = self._find_real_numbers('mos', milestone)

        for key in real_numbers.keys():
            real_numbers[key] += real_numbers_mos[key]

        initial_date = (datetime.date.today() -
                        datetime.timedelta(days=120)).isoformat()

        chart_numbers = self._get_chart_last_day_data(
            '/api/charts/product/release_chart_trends/%s/'
            'get_data?initial_date=%s' % (milestone, initial_date))

        self._verify_numbers(real_numbers, chart_numbers)

    def test_fuelmos_61_chart(self):
        self._test_fuelmos_chart('6.1')

    def test_fuelmos_70_chart(self):
        self._test_fuelmos_chart('7.0')
