import bs4

import test_app


class TestCustomReport(test_app.TestApp):
    def _get_lst_from_project(self, project_name, milestone):
        resp = self.send_get('https://launchpad.net/%s/+milestone/%s' %
                             (project_name, milestone))

        doc = bs4.BeautifulSoup(resp.text)

        table = doc.find(id='milestone_bugtasks')

        result = []
        for tr in table.tbody.find_all('tr'):
            tds = tr.find_all('td')
            bug_id = tds[1].text[1:]
            bug_name = tds[2].a.text

            self.id_map[bug_id] = bug_name
            result.append(bug_id)

        return result

    def _get_lst_from_app(self, milestone):
        url = self.make_url('/reports/custom_report?milestone=%s' % milestone)
        resp = self.send_get(url)
        doc = bs4.BeautifulSoup(resp.text)

        tables = doc.find_all('tbody')

        result = []
        for table in tables:
            trs = table.find_all('tr')

            for tr in trs:
                tds = tr.find_all('td')
                bug_id = tds[0].a.text.strip()
                bug_name = tds[1].a.text.strip()

                self.id_map[bug_id] = bug_name
                result.append(bug_id)

        return result

    def _print_bug(self, msg, bug_id):
        bug_name = self.id_map[bug_id]
        print('%-14s #%s "%s"' % (msg, bug_id, bug_name))

    def _test_report(self, milestone):
        print("Testing MOS+Fuel %s Custom report" % milestone)
        self.id_map = {}

        lp = set(self._get_lst_from_project('mos', milestone) +
                 self._get_lst_from_project('fuel', milestone))

        app = set(self._get_lst_from_app(milestone))

        for bug_id in lp - app:
            self._print_bug('Missing bug:', bug_id)

        for bug_id in app - lp:
            self._print_bug('Excessive bug:', bug_id)

    def test_61_report(self):
        self._test_report('6.1')

    def test_70_report(self):
        self._test_report('7.0')
