import unittest

from lpreports.launchpad import criterias
from test_filters import Bug


class CriteriasTestCase(unittest.TestCase):
    def setUp(self):
        self.app = self._create_app()

    def _create_app(self):
        from lpreports import application

        return application.create_app("lpreports.config.Testing")

    def test_business_days_calc(self):
        pass

    def _create_triage_criteria(self):
        owners = {
            "mos": "MOS OpenStack",
            "fuel": "Fuel"
        }
        return criterias.NonTriaged(5,
                                    ["MOS OpenStack", "Fuel"],
                                    owners,
                                    "MOS OpenStack")

    # FIXME: WARNING: requires access to pre-populated "assignees" database
    def test_triage_by_incomplete_date(self):
        with self.app.app_context():
            bug = Bug(
                assignee="mos-neutron",
                status="Incomplete",
                date_incomplete=criterias.calendar_days_ago(6)
            )

            triage = self._create_triage_criteria()
            self.assertTrue(triage.apply(bug))

            bug.date_incomplete = criterias.calendar_days_ago(1)
            self.assertFalse(triage.apply(bug))

    def test_triage_by_target_project(self):
        with self.app.app_context():
            bug = Bug(
                status="New",
                target_name="mos",
                assignee=None
            )

            triage = self._create_triage_criteria()
            self.assertTrue(triage.apply(bug))

            # Bug should be triaged by other team (Fuel team)
            bug.assignee = "vkuklin"
            self.assertFalse(triage.apply(bug))

    def test_triage_by_milestone(self):
        with self.app.app_context():
            bug = Bug(
                milestone=None,
                target_name="mos",
                importance="Critical",
                assignee="someone"
            )
            triage = self._create_triage_criteria()

            self.assertTrue(triage.apply(bug))

            bug.milestone = "1.0"
            self.assertFalse(triage.apply(bug))

    def test_triage_by_importance(self):
        with self.app.app_context():
            bug = Bug(
                milestone="1.0",
                assignee="mos-nova",
                importance="Undecided",
            )
            triage = self._create_triage_criteria()

            self.assertTrue(triage.apply(bug))

            bug.importance = "High"
            self.assertFalse(triage.apply(bug))
