import unittest

from lpreports.utils import common as utils


class WeekdayTestCase(unittest.TestCase):
    def test_lowercase_names(self):
        self.assertEqual(utils.Weekday.by_name('monday'), 0)
        self.assertEqual(utils.Weekday.by_name('tuesday'), 1)
        self.assertEqual(utils.Weekday.by_name('wednesday'), 2)
        self.assertEqual(utils.Weekday.by_name('thursday'), 3)
        self.assertEqual(utils.Weekday.by_name('friday'), 4)
        self.assertEqual(utils.Weekday.by_name('saturday'), 5)
        self.assertEqual(utils.Weekday.by_name('sunday'), 6)

    def test_uppercase_names(self):
        self.assertEqual(utils.Weekday.by_name('MONDAY'), 0)
        self.assertEqual(utils.Weekday.by_name('TUESDAY'), 1)
        self.assertEqual(utils.Weekday.by_name('WEDNESDAY'), 2)
        self.assertEqual(utils.Weekday.by_name('THURSDAY'), 3)
        self.assertEqual(utils.Weekday.by_name('FRIDAY'), 4)
        self.assertEqual(utils.Weekday.by_name('SATURDAY'), 5)
        self.assertEqual(utils.Weekday.by_name('SUNDAY'), 6)

    def test_mixedcase_names(self):
        self.assertEqual(utils.Weekday.by_name('Monday'), 0)
        self.assertEqual(utils.Weekday.by_name('Tuesday'), 1)
        self.assertEqual(utils.Weekday.by_name('Wednesday'), 2)
        self.assertEqual(utils.Weekday.by_name('Thursday'), 3)
        self.assertEqual(utils.Weekday.by_name('Friday'), 4)
        self.assertEqual(utils.Weekday.by_name('Saturday'), 5)
        self.assertEqual(utils.Weekday.by_name('Sunday'), 6)

    def test_wrong_value_passed(self):
        self.assertRaises(ValueError, utils.Weekday.by_name, 'mon')
        self.assertRaises(ValueError, utils.Weekday.by_name, 'FRI')
        self.assertRaises(ValueError, utils.Weekday.by_name, '123')
        self.assertRaises(ValueError, utils.Weekday.by_name, 123)

    def test_exception_msg(self):
        self.assertRaisesRegexp(
            ValueError,
            "^mon is an invalid week day name.*",
            utils.Weekday.by_name,
            'mon'
        )
